<?php

function idsSnort() {

    // Load snort module
    $objModule = new myModule();
    $objModule->loadModule("1");
	
    // Check if snort module is enabled
    if ($objModule->getModuleVar("enabled") == 1) {

        procWrite("A", ".. Processing Snort Sensors");
                
        // Instantiate new snortPort object
        $sp = new snortPort();
        
        // Create Sensor Object
        $objSensor = new mySensor();
        
        // Retrieve array of sensor id's
        $id = $objSensor->getSensorIDs(1);
        
        // Loop through each activated sensor
        for ($i=0;$i<count($id);$i++) {
            
        	$count = 0;
        	
            // Load sensor information
            $objSensor->loadSensor($id[$i]);
            
            // Sensor Display
            procWrite("A", " . " . ops("site_consoleName") . " Sensor ID: $id[$i] (Snort ID: " . $objSensor->getSensorVar("sid") . ")");
            procWrite("A", " . . Sensor Name: " . $objSensor->getSensorVar("name"));
            
            // Get id of last event insert
            $eStart = $objSensor->getSensorVar("lastEvent");
            if (!$eStart) $eStart = 0;
            
            // Set timers
            $timerStart = date("U");
        
            procWrite("A", " . . Last Processed Tracker: $eStart");
                
            // Perform import of snort to Aanval based on greater than stored timestamp and sensor id
            $data = $sp->processSnortRange($objSensor->getSensorVar("sid"), $id[$i], $eStart, ops("site_processorEventRun"), ops("site_snortTruncate"), 0);
        
            $lastProcessed = $data[0];
            $count = $data[1];
            $cid = $data[2];
            
            if ($cid) {
                // Perform import of snort to Aanval based on equal to stored timestamp and sensor id (This is a clean up operation to ensure all events for this timestamp have been converted)
                $data = $sp->processSnortRange($objSensor->getSensorVar("sid"), $id[$i], $lastProcessed, ops("site_processorEventRun"), ops("site_snortTruncate"), 1, $cid);			
        
                $lastProcessed = $data[0];
                $count = $data[1] + $count;
            }
                
            // Update sensor lastEvent processed
            $objSensor->setSensorVar("lastEvent", $lastProcessed);
        
            // Calculate timers
            $timerDiff = date("U") - $timerStart;
            
            // End snort processing for this sensor
            procWrite("A", " . . Imported ~$count Events in $timerDiff seconds");
            }
		
	} else {
            procWrite("A", ".. Snort Module Not Active");       
	}
}

function idsSyslog() {
	global $idsDB;

    // Load syslog module
	$objModule = new myModule();
	$objModule->loadModule("2");
	
	// Check if syslog module is enabled
	if ($objModule->getModuleVar("enabled") == 1) {	
	
		/* Syslog Daemon Processing */
		
		procWrite("A", ".. Processing Syslog Sensors");	    

		// Instantiate new processor object
	    $processor = new processor();
	
	    // Retrieve List of syslog sensor files from /syslog/ directory
	    $syslogFiles = $processor->getSyslogFiles();
	    
	    // Check for new syslog sensors (create new sensor if necessary)
	    $processor->newSyslogSensorCheck($syslogFiles);
	    
	    // Process syslog sensors (single or multiple)
	    $processor->processSyslogSensors();

	    /* Syslog Source File Processing */

		// Set variables
	    $start = date("U");
		
		// Set array of sources
		$queryA = $idsDB->query("select id, path from idsSensor where enabled = 1");
		while (list($q_id, $q_path) = mysql_fetch_row($queryA)) {
			if (strstr($q_path, "/")) $enabledSourceArray[] = $q_id;
		}
		
		if (count($enabledSourceArray) == 0) return;
		
		procWrite("A", ".. Processing File Sources (x" . count($enabledSourceArray) . ")");
		
		// Loop through sources
		foreach ($enabledSourceArray as $sid) {
	
			// Load sensor data
			$objSensor = new mySensor();
			$objSensor->loadSensor($sid);
			
			procWrite("A", " . Processing: " . $objSensor->getSensorVar("path"));
			procWrite("A", " . . Sensor Name [ID]: " . $objSensor->getSensorVar("name") . " [$sid]");
			
			// Error checking
			if (!@file_exists($objSensor->getSensorVar("path"))) {
				procWrite("A", "ERROR: Source File: " . $objSensor->getSensorVar("path") . " does not exist / cannot be found");
				continue;
			} else if (!@is_file($objSensor->getSensorVar("path"))) {
				procWrite("A", "ERROR: Source File: " . $objSensor->getSensorVar("path") . " is not a file (a directory maybe?)");
				continue;
			}
			
			// Open file handle
			$handle = @fopen($objSensor->getSensorVar("path"), "r");
			
			// Check filesize against tracker for rotated or cleared logs
			$fileInfo = fstat($handle);
			if ($objSensor->getSensorVar("tracker1") > $fileInfo[7]) {
				$objSensor->setSensorVar("tracker1", 0);
			}		
			
			// Seek to last position
			fseek($handle, $objSensor->getSensorVar("tracker1"));
			
			procWrite("A", " . . Filesize: " . humanReadableFilesize(filesize($objSensor->getSensorVar("path"))) . " seeking to: " . $objSensor->getSensorVar("tracker1"));
	
			// Begin processing
			$line = 0;
			if ($handle) {
				
				// Loop through source line by line
				while (!feof($handle)) {
					$buffer = rtrim(fgets($handle, 4096));
					$tracker1 = ftell($handle);
	
					// If line not blank
					if ($buffer) {
						$line++;
						$dataArray[] = "$buffer";
						$tracker2 = $buffer;
						
						// Exit if processing more than X lines
						if ($line >= ops("site_processorEventRun")) break;
					}
				}
				
				// Close file handle
				fclose($handle);
				
				// Store data
				if ($dataArray) $processor->processSyslogSourceFile($sid, $objSensor->getSensorVar("sid"), $dataArray);
			}
	
			// Get trackers in case no data processed
			if (!$tracker1) $tracker1 = $objSensor->getSensorVar("tracker1");
			if (!$tracker2) $tracker2 = $objSensor->getSensorVar("tracker2");
	
			// Calculate byte diff
			$byteDiff = $tracker1 - $objSensor->getSensorVar("tracker1");
			
			// Set trackers
			$objSensor->setSensorVar("tracker1", $tracker1);
			$objSensor->setSensorVar("tracker2", addslashes($tracker2));
			
			procWrite("A", " . . Processed " . humanReadableFilesize($byteDiff) . " in $line lines");
			
			// Unset used variables
			unset($dataArray, $buffer, $tracker1, $tracker2, $byteDiff);	
		}
	
		// Time calculations
	    $diff = date("U") - $start;
	
	    procWrite("A", " . Processing Syslog Complete in $diff seconds");
	    
		return $out;
	
	} else {
        procWrite("A", ".. Syslog Module Not Active");       
	}
}

function tableIntegrity() {
    global $idsDB;

    procWrite("A", " . Table Integrity Checking Started");
    
    # idsTracker - Check to ensure only 1 entry for each store/type exists
    	// Type 1 (Datastore)
		$idsQueryA = $idsDB->query("select id from idsTracker where store = '" . ops("site_activeDataStore") . "' and type = 1");
		while (list($db_col1) = mysql_fetch_row($idsQueryA)) {
			$idArray[] = $db_col1;
		}

		// Found more than 1
		if (count($idArray) >= 2) {
			if ($idArray[0] >= 1) $idsQueryA = $idsDB->query("delete from idsTracker where type = 1 and store = '" . ops("site_activeDataStore") . "' and id != '$idArray[0]'");
			
			dblog("Processor A", "Table Integrity Check", $flag, "Resolved idsTracker Duplicate Type 1");
		    procWrite("A", " . Duplicate idsTracker Resolved Type 1");
			
		}
		unset($idArray);
		
    	// Type 2 (Indexstore)
		$idsQueryA = $idsDB->query("select id from idsTracker where store = '" . ops("site_activeDataStore") . "' and type = 2");
		while (list($db_col1) = mysql_fetch_row($idsQueryA)) {
			$idArray[] = $db_col1;
		}

		// Found more than 1
		if (count($idArray) >= 2) {
			if ($idArray[0] >= 1) $idsQueryA = $idsDB->query("delete from idsTracker where type = 2 and store = '" . ops("site_activeDataStore") . "' and id != '$idArray[0]'");
			
			dblog("Processor A", "Table Integrity Check", $flag, "Resolved idsTracker Duplicate Type 2");
		    procWrite("A", " . Duplicate idsTracker Resolved Type 2");
		}
		unset($idArray);		
		
    procWrite("A", " . Table Integrity Checking Comnplete");

}

function hostnameProcessor() {

	// Return if hostname lookups disabled
	if (ops("site_hostnameLookups") != 1) return;

	$indexObj = new indexer();
	$dataStoreObj = new datastore();
	
	$hostnameRunLimit = ops("site_hostnameRunLimit");
	
	procWrite("B", ".. Hostname Processing Started");
	procWrite("B", " . Hostname Run Limit: $hostnameRunLimit");
	
	$eventTracker = $dataStoreObj->getCurrentEventID();

	$cntNew = 0;
	$cntOld = 0;

	if ($eventTracker) {
	
		for ($i=0;$i<$hostnameRunLimit;$i++) {
	
			$id = rand(1,$eventTracker);
			
			$eventData = $dataStoreObj->getEventData($id);
			if (is_array($eventData)) {
				
				if (!$eventData["src_hostname"]) {
					
					if ($eventData["src"]) {
						
						$file = $indexObj->getVar("storePath") . "/sip/" . $indexObj->getIndexFileByNumber($eventData["src"], 1);
						$file = str_replace("-1.idx", ".name", $file);
						if (!file_exists($file)) {
							$src_hostname = gethostbyaddr(acid_to_ip($eventData["src"]));
							
					        if ($fp = @fopen($file, "w")) {
					        	fputs ($fp, "$src_hostname");
					            fclose($fp);
					        }
					        
							$cntNew++;
						} else {
							$src_hostname = trim(@file_get_contents($file));
							$dataStoreObj->alterDataStore($id, "src_hostname", $src_hostname);
							
							$cntOld++;
						}
					}
				}
				if (!$eventData["dst_hostname"]) {
					if ($eventData["dst"]) {
						
						$file = $indexObj->getVar("storePath") . "/dip/" . $indexObj->getIndexFileByNumber($eventData["dst"], 1);
						$file = str_replace("-1.idx", ".name", $file);
						if (!file_exists($file)) {
							$dst_hostname = gethostbyaddr(acid_to_ip($eventData["dst"]));
							
					        if ($fp = @fopen($file, "w")) {
					        	fputs ($fp, "$dst_hostname");
					            fclose($fp);
					        }
					        
							$cntNew++;
						} else {
							$dst_hostname = trim(@file_get_contents($file));
							$dataStoreObj->alterDataStore($id, "dst_hostname", $dst_hostname);
							
							$cntOld++;
						}
					}
				}
			}
		}
	}

	procWrite("B", " . Resolved $cntNew New Hostnames");
	procWrite("B", " . Updated $cntOld SRC/DST Hostnames");
	procWrite("B", " . Hostname Processing Finished");
	
}

function deleteDataProcessor() {
	
	$indexObj = new indexer();
	
	procWrite("B", ".. Delete Processing Started");
	
	if ($tmpA_dir = @dir($indexObj->getVar("storePath") . "/delete")) {
		while ($file = $tmpA_dir->read()) {
	    	if ($file != "." && $file != "..") {
	
	    		if (strstr($file, ".evt")) {
	    			$tmpArray = explode(",", @file_get_contents($indexObj->getVar("storePath") . "/delete/$file"));
	    			break;
	    		}
	    	}
		}
	}
	
	if ($tmpArray) {
		
		// Process here
		$cnt = count($tmpArray);
		
		procWrite("B", " . Found Events for Deletion");
		procWrite("B", " . Events: $cnt");
		
		$dataStoreObj = new datastore();
		
		foreach ($tmpArray as $id) {
		
			if ($dataStoreObj->removeData($id) == 0) {
				$problem = 1;
			}
		}

		@unlink($indexObj->getVar("storePath") . "/delete/$file");
		
		procWrite("B", " . Events Deleted");
		
	} else {

		procWrite("B", " . Nothing to Delete");
		
	}

	if ($problem) procWrite("B", " . Problem: Events Not Being Deleted");
	
	procWrite("B", " . Delete Processing Finished");
}


function deleteIndexProcessor() {
	
	$indexObj = new indexer();
	$dataStoreObj = new datastore();
	
	procWrite("C", ".. Index Deletion Started");
	
	if ($tmpA_dir = @dir($indexObj->getVar("storePath") . "/delete")) {
		while ($file = $tmpA_dir->read()) {
	    	if ($file != "." && $file != "..") {
	
	    		if (strstr($file, ".dat")) {
	    			$tmpArray = explode(",", @file_get_contents($indexObj->getVar("storePath") . "/delete/$file"));
	    			break;
	    		}
	    	}
		}
	}
	
	if ($tmpArray) {
		
		// Process here
		$cnt = count($tmpArray);
		
		procWrite("C", " . Found Indexes for Deletion");
		procWrite("C", " . Events: $cnt");
		
		foreach ($tmpArray as $id) {
		
			$data = $dataStoreObj->getEventData($id);
			
			if ($indexObj->removeIndex($id, "sip", "src", $data) == 0) { $problem = 1; }
			if ($indexObj->removeIndex($id, "dip", "dst", $data) == 0) { $problem = 1; }
			if ($indexObj->removeIndex($id, "sport", "sport", $data) == 0) { $problem = 1; }
			if ($indexObj->removeIndex($id, "dport", "dport", $data) == 0) { $problem = 1; }
			if ($indexObj->removeIndex($id, "sensor", "sensorid", $data) == 0) { $problem = 1; }
			if ($indexObj->removeIndex($id, "protocol", "ip_proto", $data) == 0) { $problem = 1; }
			if ($indexObj->removeIndex($id, "level", "sig_priority", $data) == 0) { $problem = 1; }
			if ($indexObj->removeIndex($id, "module", "module", $data) == 0) { $problem = 1; }
			if ($indexObj->removeIndex($id, "signature", "sig_sid", $data) == 0) { $problem = 1; }
			
			$wordString = $data["sig_class_name"] . " " . $data["sig_name"];
			if (strlen($wordString) > 1) {
	        	$array1 = split("[^[:alpha:]]+", strtolower($wordString));
	        	$array1 = array_unique($array1);
	        	
	        	foreach ($array1 as $word) {
	        		$data["word"] = $word;
					if ($indexObj->removeIndex($id, "word", "word", $data) == 0) { $problem = 1; }
	        	}
			}
		}

		$newFile = str_replace(".dat", ".evt", $file);
		
		rename($indexObj->getVar("storePath") . "/delete/$file", $indexObj->getVar("storePath") . "/delete/$newFile");
		
		procWrite("C", " . Indexes Deleted");
		
	} else {

		procWrite("C", " . Nothing to Delete");
		
	}

	if ($problem) procWrite("B", " . Problem: Some Indexes Not Being Deleted");
	
	procWrite("C", " . Index Deletion Finished");
}

function writeFlashPoll($cnt) {
	global $root_path;
	
	$cnt = $cnt / 3;
	$cnt = number_format($cnt);
	
	if ($cnt > 125) $cnt = 125;
	
	// Attempt to read in flash poll status file
	if (@file_exists("$root_path" . "poll/flash_poll.sts")) {
	
		// Flash poll status file exists
		
		// Read file into array
		$fileData = file("$root_path" . "/poll/flash_poll.sts");
		
		// Trim and append new count data
		$fileData = trim($fileData[0]) . ",$cnt";
		
		// Replace array with new array which includes new count data
		$fileData = explode(",", $fileData);
		
		// Slice out the last elements
		$fileData = array_slice($fileData, -75);

		// Put the last 10 elements back in an array
		foreach ($fileData as $data) {
			$out .= "$data,";
		}
		
		// Trim ending comma
		$out = substr($out, 0, -1);
		
		// Open file, truncate and write new comma delimited list
	    if ($fp = @fopen("$root_path" . "poll/flash_poll.sts", "w")) {
	
	    	// Write data
	    	fputs ($fp, "$out");
	    	
	    	// Close file handle
	        fclose($fp);
	    }			
		
	} else {
		
		// Flash poll status file does not exist

		// Open new file
	    if ($fp = @fopen("$root_path" . "poll/flash_poll.sts", "w")) {
	
	    	// Write procWrite data
	    	fputs ($fp, "$cnt");
	    	
	    	// Close file handle
	        fclose($fp);
	    }	

	}
	
	procWrite("A", ".. Flash Heartbeat Monitor Updated");
}

function writeImportCalculator($cnt, $second) {
	global $root_path;

	if ($fp = @fopen("$root_path" . "/poll/eventImport.sts")) {
		$fileInfo = fstat($fp);
		if ($fileInfo[7] >= 10240) {
			$remove = 1;
		}
		fclose($fp);
		
		if ($remove) @unlink("$root_path" . "/poll/eventImport.sts");
	}
	
	// Attempt to read in flash poll status file
	if (@file_exists("$root_path" . "/poll/eventImport.sts")) {
	
		// Flash poll status file exists
		
		// Read file into array
		$fileData = file("$root_path" . "/poll/eventImport.sts");
		
		$fileData[10] = trim($fileData[9]);
		$fileData[9] = trim($fileData[8]);
		$fileData[8] = trim($fileData[7]);
		$fileData[7] = trim($fileData[6]);
		$fileData[6] = trim($fileData[5]);
		$fileData[5] = trim($fileData[4]);
		$fileData[4] = trim($fileData[3]);
		$fileData[3] = trim($fileData[2]);
		$fileData[2] = trim($fileData[1]);
		$fileData[1] = trim($fileData[0]);
		$fileData[0] = $cnt . ":$second";

		$out = implode("\n", $fileData);
		
		// Open file, truncate and write new comma delimited list
	    if ($fp = @fopen("$root_path" . "/poll/eventImport.sts", "w")) {
	
	    	// Write data
	    	fputs ($fp, "$out");
	    	
	    	// Close file handle
	        fclose($fp);
	    }			
		
	} else {
		
		// Flash poll status file does not exist

		// Open new file
	    if ($fp = @fopen("$root_path" . "/poll/eventImport.sts", "w")) {
	
	    	// Write procWrite data
	    	fputs ($fp, "$cnt");
	    	
	    	// Close file handle
	        fclose($fp);
	    }	

	}
	
	procWrite("A", ".. Event Calculator Updated");
}

function gatherSystemStatus($type, $mode = "w") {
	global $root_path, $masterCnt;

	if (ops("site_systemStatus") == 0) return;
	
    procWrite("B", ".. System Status: $type");

    switch ($type) {
    	case "uptime":
    		$data = trim(`uptime`);
    		break;

    	case "load_history":
    		$data = date("U") . " " . trim(`uptime`) . "\n";
    		break;
    		
    	case "df":
    		$data = trim(`df -h`);
    		break;
    		
    }

    if ($data) {
    	
    	if ($mode == "a" && in_array($masterCnt, array(1))) return;
    	
    	if (!file_exists("$root_path/tmp/ss")) @mkdir("$root_path/tmp/ss");
    	$fp = fopen("$root_path/tmp/ss/$type.sts", "$mode");
    	fwrite($fp, $data);
    	fclose($fp);
    	
    	
    	if ($mode == "a") {
	    	if ($fp = @fopen("$root_path/tmp/ss/$type.sts", "r")) {
				$fileInfo = fstat($fp);
				if ($fileInfo[7] >= 256000) {
					$move = 1;
				}
				fclose($fp);
	    	}
	    	if ($move == 1) rename("$root_path/tmp/ss/$type.sts", "$root_path/tmp/ss/$type." . date("U") . ".old.sts");
    	}
    	
    }
    
}

function permissionsCheck() {
	global $root_path;

	// Continue only if permissions checking is enabled
	if (ops("site_permChecking") == 0) return;
	
	// Randomly perform permissions checking (save cpu and disk usage)
	if (rand(1,35) != 17) return;

    procWrite("B", ".. Permissions Checking Started");

    procWrite("B", " . /" . "conf");
    if (!file_exists("$root_path" . "conf")) @mkdir("$root_path" . "conf");
    if (!@chmod("$root_path" . "conf", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "conf");
    }    

    procWrite("B", " . /" . "poll/tmp");
    if (!file_exists("$root_path" . "poll/tmp")) @mkdir("$root_path" . "poll/tmp");
    if (!@chmod("$root_path" . "poll/tmp", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "poll/tmp");
    }
    
    procWrite("B", " . /" . "poll/sensorData");
    if (!file_exists("$root_path" . "poll/sensorData")) @mkdir("$root_path" . "poll/sensorData");
    if (!@chmod("$root_path" . "poll/sensorData", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "poll/sensorData");
    }
    
    procWrite("B", " . /" . "tmp");
    if (!file_exists("$root_path" . "tmp")) @mkdir("$root_path" . "tmp");
    if (!@chmod("$root_path" . "tmp", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "tmp");
    }

    procWrite("B", " . /" . "tmp/sc");
    if (!file_exists("$root_path" . "tmp/sc")) @mkdir("$root_path" . "tmp/sc");
    if (!@chmod("$root_path" . "tmp/sc", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "tmp/sc");
    }

    procWrite("B", " . /" . "version");
    if (!file_exists("$root_path" . "version")) @mkdir("$root_path" . "version");
    if(!@chmod("$root_path" . "version", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "version");
    }

    procWrite("B", " . /" . "syslog");
    if (!file_exists("$root_path" . "syslog")) @mkdir("$root_path" . "syslog");
    if(!@chmod("$root_path" . "syslog", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "syslog");
    }
   
    procWrite("B", " . /" . "installation");
    if (!file_exists("$root_path" . "installation")) @mkdir("$root_path" . "installation");
    if(!@chmod("$root_path" . "installation", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "installation");
    }    
    
    procWrite("B", " . /" . "language");
    if (!file_exists("$root_path" . "language")) @mkdir("$root_path" . "language");
    if(!@chmod("$root_path" . "language", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "language");
    }    
    
    procWrite("B", " . Permissions Checking Finished");

}

function remoteVersionCheck() {
    global $idsDB, $root_path;

    /* Checks the Aanval.com version website for new available builds */
    
    // Start version checking at least once upon each install / upgrade
    if (@file_exists("$root_path" . "version/version.one")) {
		ops("site_versionChecking", "1");    	
		@unlink("$root_path" . "version/version.one");
    }

    // Check if version checking is enabled
    if (ops("site_versionChecking") == 1) {

        procWrite("B", ".. Version Checking Started");
    	
    	// Retrieve unix epoch
        $epoch = date("U");
        
        // Read in current version /version/verstion.txt file if available
        if ($tmp = @file("$root_path/version/version.txt")) {

        	// Create various variables from array
        	$major = rtrim($tmp[0]);
        	$minor = rtrim($tmp[1]);
        	$name = rtrim($tmp[2]);
        	$build = rtrim($tmp[3]);
        	
        	// Read in available version /version/ver file if available
	        if ($tmp = @file("$root_path/version/ver")) {
	        	
	        	// Create $time variable from array
	        	$time = rtrim($tmp[4]);
	
		        // Check $time variable to see if 15 minutes has passed and it is time to recheck for new versions
	        	if (($time + 900) < $epoch) {
	
		        	// Retrieve site registration code
		            $key1 = ops("site_registration");
		            
		        	procWrite("B", " . Connecting: update.aanval.com");
	
	        		// Make outbound web connection to aanval server
		        	$versionData = exConnect("update.aanval.com", "updater/aanval_ver.php?major=$major&minor=$minor&build=$build&certify=$key1&ver=2&sub=$sub");

		            // Disable Version Checking if $versionData is empty
		            if ($versionData == "") {
		            	procWrite("B", " ! Version Data Empty (connectivity problem)");
		            }

		            if (!$sub && $versionData != "") {          	
		            	// Create /version/ver file handle (this is the available version file)
			            if ($fp = @fopen("$root_path/version/ver", "w")) {
			            	
							procWrite("B", " . Writing /version/ver tracker");
			            	
		            		// Write web retrieved version data to /version/ver
		                    fputs ($fp, "$versionData\n$epoch");
		                    
		                    // Close file handle
		                    fclose($fp);
			            }
		            }
		            
	        	} else {
	        		
		        	// Calculate seconds until we make version check
		        	$until = $time + 900 - $epoch;
		        	
			        procWrite("B", " . Next Check in $until seconds");
	        	}
	        } else {

	        	// Create /version/ver file handle (fill with emptyness)
	            if ($fp = @fopen("$root_path/version/ver", "w")) {
	            	
					procWrite("B", " . Writing /version/ver tracker");
	            	
            		// Write web retrieved version data to /version/ver
                    fputs ($fp, "0\n0\n0\n0\n0\n:");
                    
                    // Close file handle
                    fclose($fp);
	            }	        	
	        }
        } else {
        	
    		// Could not open current version file: /version/version.txt    	
        	procWrite("B", " ! File Error: $root_path/version/version.txt could not be read");
        	procWrite("B", " ! Version Checking automatically disabled");
        	ops("site_versionChecking", "0");
        }

    }

    /* Download Performed Here */
    
	// Check if the UpdateNow variable == 1
    if (ops("site_versionUpdateNow") == 1) {

    	// Download the .tar.gz file from Aanval - if returns 0 do not continue
        if (updateDownload("www.aanval.com", "/downloads/", "aanval-latest-stable.tar.gz") == 0) {
        
	    	loopCancel("?op=prv_versionUpdateError", "Version Update Error - Unable to Download");
    	
        } else {

			procWrite("B", " . Processing New Update");
		    loopMessage("Upgrading Console");
        	
        	// Change to the root aanval directory
            @chdir("$root_path/");
            
            // External execute: gunzip .tar.gz
            @exec("gunzip -c aanval-latest-stable.tar.gz | tar -xvf -");
            
            // External execute: rm -f to remove downloaded file
            @exec("rm -f aanval-latest-stable.tar.gz");

	    	loopCancel("?", "Version Update Successful");
            
        }

        // Disable UpdateNow
        ops("site_versionUpdateNow", "0");
    }

	procWrite("B", " . Version Checking Finished");

}

function updateDownload($host, $path, $file) {
    global $root_path;

    // Default status = 0
    $status = 0;

    procWrite("B", " . Downloading from: $host");
    procWrite("B", " . Receiving file: $file");

    loopMessage("Downloading Update ($file) from $host");
    
    // Make outbound web connection to Aanval server
    $downloadData = exConnect($host, "$path$file");
    
    // Disable Version Checking if $downloadData is empty
    if ($downloadData == "") {
    	procWrite("B", " ! Download Data is empty or not correct (might be a problem)");
    	procWrite("B", " ! Version Checking automatically disabled");
    	ops("site_versionChecking", "0");
    } else {
	    
    	// Create file handle
	    if ($handle = @fopen("$root_path/$file","w")) {
	    	
		    procWrite("B", " . Writing File: $root_path/$file");
		    
	    	// Dump data into new file handle
	    	fputs($handle, $downloadData);
	    	
	    	// Close file handle
	    	fclose($handle);
	    } else {

	    	// File could not be opened for writing
    		procWrite("B", " ! File Error: $root_path/$file could not be created");
	    	procWrite("B", " ! Version Checking automatically disabled");
	    	ops("site_versionChecking", "0");
	    }
	    
	    // Check if newly created file exists
	    if (@file_exists("$root_path/$file")) $status = 1;
    }

    return $status;
}

function recursiveModuleLoad($null = 0) {
    global $root_path;

    # Read in all available modules recursively
    if (!$this_dir = @dir("$root_path/modules/")) {
        procWrite("A", "conf.php ERROR: \$root_path not correct or $root_path/modules/ not found! Check your conf.php settings");
    } else {
        while ($dir = $this_dir->read()) {
            if ($dir != "." && $dir != "..") {
                if (substr($dir, -4, 4) == "_MOD") {
                    $this_sub = dir("$root_path/modules/$dir");
                    while ($file = $this_sub->read()) {
                        if ($file != "." && $file != "..") {
                            if (substr($file, 0,4) == "inc_") {
                                include_once "../modules/$dir/$file";
                            }
                        }
                    }
                }
            }
        }
    }
}

function actionEmail($actionid, $trackid, $toEmail) {
    global $ops, $idsDB;

    // Build action object and load master details
    $objAction = new myAction();
	$objAction->actionMasterLoad($actionid);
	
	// Set userid variable for query
    $uid = $objAction->getActionMasterVar("userid");

    // Query user details for defaults
    $idsQueryA = $idsDB->query("select email, fname, lname from idsUsers where id = '$uid'");
    while (list($d_email, $d_fname, $d_lname) = mysql_fetch_row($idsQueryA)) {
        $email = $d_email;
        $fname = $d_fname;
        $lname = $d_lname;
    }

    $dataStoreObj = new dataStore();
	$eventData = $dataStoreObj->getEventData($trackid);
    
	$module = $eventData["module"];
	$proto = $eventData["ip_proto"];
	$sip = $eventData["src"];
	$dip = $eventData["dst"];
	$sport = $eventData["sport"];
	$dport = $eventData["dport"];
	$etypeid = $eventData["sig_class"];
	$etypename = $eventData["sig_class_name"];
	$enameid = $eventData["sig_sid"];
	$ename = $eventData["sig_name"];
	$level = $eventData["sig_priority"];
	$payload = $eventData["data_payload"];
	$sensorid = $eventData["sensorid"];
	$timestamp = $eventData["timestamp"];
	
    // Set sensor name for sensorid variable
    $objSensor = new mySensor;
    $sensorid = $objSensor->getSensorName($sensorid);
	
    // Retrieve proto name by number
    if ($proto == 1) $proto = "ICMP";
    if ($proto == 6) $proto = "TCP";
    if ($proto == 17) $proto = "UDP";

    // Check if snort and decode payload
    if ($module == 1) { 
    	$payload = packet_payload($payload, 3, 1);
    }        

	// Set sender name
    if ($objAction->getActionMasterVar("senderName") == "") {
    	$fromName = "$fname $lname";
    } else {
    	$fromName = $objAction->getActionMasterVar("senderName");
    }        
    
	// Set sender email
    if ($objAction->getActionMasterVar("senderEmail") == "") {
    	$fromEmail = "$email";
    } else {
    	$fromEmail = $objAction->getActionMasterVar("senderEmail");
    }           
    
    // Set subject
    if ($objAction->getActionMasterVar("senderSubject") == "") {
    	$subject = "! " . ops("site_consoleName") . " Alert ($ename)";
    } else {

    	// Set subject and run through text replace
    	$subject = actionTextReplace($objAction->getActionMasterVar("senderSubject"), $actionid, $trackid, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp);
    }        
    
    // Set body
    if ($objAction->getActionMasterVar("senderBody") == "") {
    
    	$body = "The following alert was generated by the " . ops("site_consoleName") . " Intrusion Detection Console.\n---\n\n";
    	$body .= "Timestamp: TIMESTAMP\n\n";
    	$body .= "Risk Level: RISK_LEVEL\n\n";
	    $body .= "Source IP: SOURCE_IP : SOURCE_PORT\n";
	    $body .= "Destination IP: DESTINATION_IP : DESTINATION_PORT\n";
	    $body .= "Sensor: SENSOR\n\n";
	    $body .= "Detected Event: SIG_NAME (SIG_NAME_ID)\n\n";
	    $body .= "Detected Event Category: SIG_CLASS (SIG_CLASS_ID)\n\n";
	    $body .= "" . ops("site_consoleName") . " ID: AANVALID\n";
	    $body .= "Action ID: ACTIONID\n";
		$body .= "---\n";
	    $body .= "Payload:\n";
	    $body .= "PAYLOAD\n";
    
    	$body = actionTextReplace($body, $actionid, $trackid, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp);
    } else {
    	
    	// Set body and run through text replace
    	$body = actionTextReplace($objAction->getActionMasterVar("senderBody"), $actionid, $trackid, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp);
    }
    
    // Send email
    sendmail($fromEmail, $fromName, $toEmail, $toEmail, $subject, $body);
}

function actionTextReplace($data, $action, $id, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp) {

	/* Replaces text for emails which use keywords for replacing text */
	
	// Perform string replace on all keywords
	$data = str_replace("ACTIONID", $action, $data);
	$data = str_replace("AANVALID", $id, $data);
	$data = str_replace("PROTOCOL", $proto, $data);
	$data = str_replace("MODULE", $module, $data);
	$data = str_replace("SOURCE_IP", acid_to_ip($sip), $data);
	$data = str_replace("DESTINATION_IP", acid_to_ip($dip), $data);
	$data = str_replace("SOURCE_PORT", $sport, $data);
	$data = str_replace("DESTINATION_PORT", $dport, $data);
	$data = str_replace("SIG_NAME_ID", $enameid, $data);
	$data = str_replace("SIG_CLASS_ID", $etypeid, $data);
	$data = str_replace("SIG_NAME", $ename, $data);
	$data = str_replace("SIG_CLASS", $etypename, $data);
	$data = str_replace("RISK_LEVEL", $level, $data);
	$data = str_replace("PAYLOAD", $payload, $data);
	$data = str_replace("SENSOR", $sensorid, $data);
	$data = str_replace("TIMESTAMP", timeDisplay($timestamp, 1), $data);
	
	// Return data
	return $data;
}

function execCommand($cmd, $trackid) {
    global $ops, $idsDB;

    $dataStoreObj = new dataStore();
	$eventData = $dataStoreObj->getEventData($trackid);
    
	$module = $eventData["module"];
	$proto = $eventData["ip_proto"];
	$sip = $eventData["src"];
	$dip = $eventData["dst"];
	$sport = $eventData["sport"];
	$dport = $eventData["dport"];
	$etypeid = $eventData["sig_class"];
	$etypename = $eventData["sig_class_name"];
	$enameid = $eventData["sig_sid"];
	$ename = $eventData["sig_name"];
	$level = $eventData["sig_priority"];
	$payload = $eventData["data_payload"];
	$sensorid = $eventData["sensorid"];
	$timestamp = $eventData["timestamp"];
    
	$cmd = actionTextReplace($cmd, $actionid, $trackid, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp);
    
    dblog("ACTION_MOD", $cmd, $flag, "Command");

    $fp = popen($cmd, 'r');
    while(!feof($fp)) {
        $read .= fread($fp,1024);
    }
    pclose($fp);

}

function scanProcessor() {
    global $login, $idsDB;

    procWrite("B", ".. Scanner Started");

    $idsQueryA = $idsDB->query("SELECT id, ip, status, uid, tcp_ports, udp_ports, enabled, timestamp, type FROM idsScanner where status = 3 and enabled = 1 order by timestamp limit 10");
    for ($i = 0; $row = mysql_fetch_row($idsQueryA); $i++) {
        $id[] = $row[0];
        $ip[] = $row[1];
        $status[] = $row[2];
        $uid[] = $row[3];
        $tcp_ports[] = $row[4];
        $udp_ports[] = $row[5];
        $enabled[] = $row[6];
        $timestamp[] = $row[7];
        $type[] = $row[8];
    }

    if (!$id) procWrite("B", " . Nothing to do");

    for ($i=0;$i<count($id);$i++) {
        if ($type[$i] == 1) {
            $success = pingHost($id[$i]);
            procWrite("B", " - Pinging " . acid_to_ip($ip[$i]) . " $success");
        } else if ($type[$i] == 2 || $type[$i] == 3) {
            $success = pingHost($id[$i]);
            procWrite("B", " - Pinging " . acid_to_ip($ip[$i]) . " $success");
            procWrite("B", " - Port Scanning " . acid_to_ip($ip[$i]));
            scanHost($id[$i], $type[$i]);
        }
    }

    procWrite("B", ".. Scanner Finished");
	    
}

function communityBufferProcess() {
    global $ops, $idsDB;

    $myCommunityObj = new myCommunity();

    if (ops("site_community") == 1) {

		procWrite("B", ".. Community Processing Started");

	    $idsQueryA = $idsDB->query("select id, sender, message, timestamp from " . ops("site_table.idsCommunityBuffer") . " order by timestamp desc");
        while (list($d_id, $d_sender, $d_message, $d_timestamp) = mysql_fetch_row($idsQueryA)) {
            transmitCommunityBuffer($d_sender, $d_message);
            $myCommunityObj->communityBufferDelete($d_id);
        }

        $idsQueryA = $idsDB->query("select id from " . ops("site_table.idsCommunityMain") . " order by id desc limit 1");
        while (list($d_id) = mysql_fetch_row($idsQueryA)) {
            $id = $d_id;
	    }
	    
	    if (!$id) {
            receiveCommunityMain(0);
        } else if ($id) {
		    receiveCommunityMain($id);
	    }

	    procWrite("B", " . Community Processing Finished");
    }
}

function transmitCommunityBuffer($sender, $message) {
    global $ops;

    procWrite("B", " . Transmitting Buffer");

	// build payload
	$payload = "$sender||$message";

	// base64 payload data
	$payload = base64_encode($payload);
	$key1 = ops("site_registration");
    $out = exConnect("community.aanval.com", "community_payload.php?payload=$payload&certify=$key1");

	procWrite("B", " . Transmitted: $out");
}

function receiveCommunityMain($id) {
	global $ops, $idsDB;
	
	$epoch = date("U");

	procWrite("B", " . Downloading Messages");
	procWrite("B", " . Starting Message ID: $id");
	
	$key1 = ops("site_registration");
	$out = exConnect("community.aanval.com", "community_receive.php?id=$id&certify=$key1");
	
	// base64 payload data
	$payload = base64_decode($out);
	$payload = explode("|||", $payload);
	
	for ($i=0;$i<count($payload);$i++) {
		$data = explode("||", $payload[$i]);
		$data[1] = strip_tags(addslashes($data[1]));
		$data[2] = strip_tags(addslashes($data[2]));
		
		if (strstr($data[2], "red:")) {
			$data[2] = str_replace("red:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=darkred>$data[2]</font>";
		} else if (strstr($data[2], "orange:")) {
			$data[2] = str_replace("orange:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=darkorange>$data[2]</font>";
		} else if (strstr($data[2], "blue:")) {
			$data[2] = str_replace("blue:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=darkblue>$data[2]</font>";
		} else if (strstr($data[2], "green:")) {
			$data[2] = str_replace("green:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=darkgreen>$data[2]</font>";
		} else if (strstr($data[2], "purple:")) {
			$data[2] = str_replace("purple:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=purple>$data[2]</font>";
		}
		
		if ($data[0]) {
			$idsQueryA = $idsDB->query("insert into " . ops("site_table.idsCommunityMain") . " (id, sender, message, timestamp) values ('$data[0]', '$data[1]', '$data[2]', '$epoch')");
		}
	}
	
}

function sigProcessor($file, $type) {
    global $ops, $idsDB, $root_path;

    if ((ops("site_sigProcessorEnabled") != 1)) return;
    
    procWrite("B", ".. Processing Signatures");
    $epoch = date("U");
    $lastRun = statusSigProcessorLastRunDiff();

    $objStatus = new myStatus();
    $objStatus->loadStatus();
    $last = $objStatus->getStatusVar("sigProcessor_lastRun");

    if (ops("site_sigFileCustom") != "") {
	    $file = ops("site_sigFileCustom");
	    if(file_exists("$root_path/tmp/sig/$file")) $fileFound = 1;
    }    
    
    if ($lastRun > ops("site_sigProcessorFrequency") || ops("site_sigProcessorNow") == 1 || $fileFound == 1) {
        procWrite("B", ".. Signature System Executed");

        // Get Signature File - Save & Untar File
        if (ops("site_sigProcessorSite") == "A") {
            $primarySite = ops("site_sigProcessorSiteA");
        } else {
            $primarySite = ops("site_sigProcessorSiteB");
        }

        // Check for and / or retrieve file
        if (!$fileFound) {
        	if (sigDownload($primarySite, $file) == 0) return;
        }
        @chdir("$root_path/tmp/sig/");
        @exec("gunzip -c $file | tar -xvf -");
        @unlink("$root_path/tmp/sig/$file");

        // Process INTO signature system
        sigDataProcess();

        // Trigger Sensors to retreive new signatures
        $idsQueryA = $idsDB->query("select distinct sensorid from idsSignaturesIndex");
        while (list($d_sid) = mysql_fetch_row($idsQueryA)) {
            $sid = $d_sid;
            $idsQueryB = $idsDB->query("insert into idsSensorManageMessages (sid, message, enabled, issued) values ('$sid', 'SNORTRULES', '1', '$epoch')");
            dblog("SIG_PROCESSOR", "Snort Message Post ($sid)", 0, "SNORTRULESSEND");
            $idsQueryB = $idsDB->query("insert into idsSensorManageMessages (sid, message, enabled, issued) values ('$sid', 'SNORTRESTART', '1', '$epoch')");
            dblog("SIG_PROCESSOR", "Snort Message Post ($sid)", 0, "SNORTRESTART");
        }

        // Save Status & Finish
        if ($type == 2) {
	    	$objStatus = new myStatus();
	    	$objStatus->loadStatus();
	    	$objStatus->setStatusVar("sigProcessor_lastRun", date("U"));

	        // Update SigProcessorNow OPS Setting to 0
	        $ops->setOpsVar("site_sigProcessorNow", 0);
        }

        sendSignatureUpdateEmail($last);

        procWrite("B", " . Done!");
    } else {
        procWrite("B", " . Nothing to do");
    }

}

function sigDownload($host, $file) {
    global $root_path;

    $status = 0;

    procWrite("B", " - Downloading from:\n - $host\n - Receiving file:\n - $file");

    $data = exConnect($host, $file);
    $handle = fopen("$root_path/tmp/sig/$file","w");
    fputs($handle, $data);
    fclose($handle);

    if (file_exists("$root_path/tmp/sig/$file")) $status = 1;

    return $status;
}

function sigDataProcess() {
    global $root_path;

    $epoch = date("U");
    $ruleDir = "$root_path/tmp/sig/rules/";

    procWrite("B", " - Processing Signatures");

    if (!$this_dir = @dir("$ruleDir")) {
            procWrite("B", ".. ERROR:<p>$ruleDir not correct or not found!");
    } else {
	    while ($file = $this_dir->read()) {
	        if ($file != "." && $file != "..") {
	            if (strstr($file, ".rules")) {
	                $ruleFile = file("$ruleDir/$file");
	
	                for($i=0;$i<count($ruleFile);$i++) {
	                    sigInsert($ruleFile[$i], $file);
	                }
	            }
	        }
	    }
    }
}

function sendSignatureUpdateEmail($last) {

	$message = "Signature Updates Report:\n\n";

    $to = trim(ops("site_signatureUpdateEmail"));
    $subject = ops("site_signatureUpdateEmailSubject");
    sendmail(ops("site_signatureUpdateEmail"), "" . ops("site_consoleName") . " - Signature Updater", $to, $to, $subject, $message);
}

function analysisProcessor() {
    global $ops, $idsDB;

    $objSensor = new mySensor();
    $sensorArray = $objSensor->getSensorIDs(1);
    
    if (count($sensorArray) == 0) return;
    
    procWrite("D", ".. Analysis Processing Started");
	
    foreach ($sensorArray as $sensor) {

	    $epoch = date("U");
    	$objSensor->loadSensor($sensor);
    	
	    procWrite("D", " . Processing Sensor: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]");
    	
	    ### Sensor Status ###
	    
    	if ($objSensor->getSensorVar("countStatusFailure") >= 5 && $objSensor->getSensorVar("countStatusFailure") <= 10) {

    		// Update status count to bypass this section
    		$objSensor->setSensorVar("lastStatusFailure", $epoch);
    		
    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Restart Attempted: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Problem:\n";
				$message .= "Snort appears to have stopped on sensor '" . $objSensor->getSensorVar("name") . "'.\n\n";
				$message .= "Solution:\n";
                $message .= "The console attempted to restart snort and if successful, you will not receive any further emails. ";
                $message .= "However, if this snort restart fails you will receive additional updates via email and ";
                $message .= "should investigate the problem as soon as possible.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 11);

    		// Issue snort restart
    		sensorMessagePost("SNORTRESTART", $sensor);
    		
    	} else if ($objSensor->getSensorVar("countStatusFailure") >= 30 && $objSensor->getSensorVar("countStatusFailure") <= 40) {

    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Status Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 1 of 4:\n---\n\nProblem:\n";
				$message .= "A problem with snort sensor '" . $objSensor->getSensorVar("name") . "' has been detected. ";
                $message .= "This usually indicates the snort process is no longer running on the mentioned sensor.\n\n";
                $message .= "The console has already attempted an automatic restart of snort, however it appears to have failed.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will attempt to restart snort again momentarily, if this fails you will ";
                $message .= "receive an additional warning by email and should begin resolving the problem as quickly as possible.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Issue snort restart
    		sensorMessagePost("SNORTRESTART", $sensor);
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 41);    		
    		
    	} else if ($objSensor->getSensorVar("countStatusFailure") >= 100 && $objSensor->getSensorVar("countStatusFailure") <= 110) {

    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Status Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 2 of 4:\n---\n\nProblem:\n";
				$message .= "The problem with snort sensor '" . $objSensor->getSensorVar("name") . "' has not been resolved. ";
                $message .= "The snort process appears to no longer be running or the console can no longer detect its presence.\n\n";
                $message .= "The console has already attempted an automatic restart of snort twice, however both attempts appear to have ";
                $message .= "failed and this situation requires human intervention to be resolved.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will continue to monitor snort and wait for the situation to be resolved. If this problem is not ";
                $message .= "resolved within the next 15 - 20 minutes, additional warning emails will be sent as a reminder.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 111); 
    		    		
    	} else if ($objSensor->getSensorVar("countStatusFailure") >= 460 && $objSensor->getSensorVar("countStatusFailure") <= 470) {

    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Status Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 3 of 4:\n---\n\nProblem:\n";
				$message .= "The problem with snort sensor '" . $objSensor->getSensorVar("name") . "' remains to be unresolved. ";
                $message .= "The snort process appears to no longer be running or the console can no longer detect its presence.\n\n";
                $message .= "This is a reminder by email that this issue requires human intervention.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will attempt to restart snort one last time, to assist with this problem. However, the console ";
                $message .= "will continue to monitor snort and wait for the situation to be resolved. If this problem is not ";
                $message .= "resolved within the next 15 - 20 minutes, one last email warning will be sent as a reminder.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Issue snort restart
    		sensorMessagePost("SNORTRESTART", $sensor);
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 471);     		
    		
    	} else if ($objSensor->getSensorVar("countStatusFailure") >= 820 && $objSensor->getSensorVar("countStatusFailure") <= 830) {

    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Status Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 4 of 4:\n---\n\nProblem:\n";
				$message .= "This is your last reminder / warning email.\n\n";
				$message .= "The problem with snort sensor '" . $objSensor->getSensorVar("name") . "' remains to be unresolved. ";
                $message .= "The snort process appears to no longer be running or the console can no longer detect its presence. The process ";
				$message .= "has not been running for quite some time now and needs to be resolved immediately. ";
                $message .= "This is a reminder by email that this issue requires human intervention.\n\n";
				$message .= "Solution:\n";
                $message .= "The console has attempted to restart snort several times and appears to have failed. Human intervention is ";
                $message .= "required at this point in time. This is your last email concerning this sensor status issue at this time.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 831);
    	}
    	
    	### Comm ###

		if (($epoch - 30) > $objSensor->getSensorVar("hb")) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);
    		$objSensor->setSensorVar("countCommFailure", $objSensor->getSensorVar("countCommFailure") + 1);
		} else {
    		$objSensor->setSensorVar("countCommFailure", 0);
		}
    	
		if ($objSensor->getSensorVar("countCommFailure") >= 10 && $objSensor->getSensorVar("countCommFailure") <= 20) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);

    		// Email
    		if ($objSensor->getSensorVar("emailOnCommFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnCommFailure"));
				$subject = "SMT Communication Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 1 of 4:\n---\n\nProblem:\n";
				$message .= "A communication issue with sensor '" . $objSensor->getSensorVar("name") . "' and the console has been detected. ";
                $message .= "This usually indicates a network connectivity related issue has occurred or the sensor management tools ";
                $message .= "have stopped.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will continue to monitor the progress of this issue, however this should be reviewed as soon as ";
                $message .= "possible in the event there is a serious network or system problem.\n\n";
                $message .= "Additional email warnings will be sent shortly if this problem is not resolved.\n\n";
                $message .= "---\nCommunication Failure Count: " . $objSensor->getSensorVar("countCommFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}    		
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countCommFailure", 21);
    		
		} else if ($objSensor->getSensorVar("countCommFailure") >= 100 && $objSensor->getSensorVar("countCommFailure") <= 110) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);

    		// Email
    		if ($objSensor->getSensorVar("emailOnCommFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnCommFailure"));
				$subject = "SMT Communication Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 2 of 4:\n---\n\nProblem:\n";
				$message .= "A communication issue with sensor '" . $objSensor->getSensorVar("name") . "' remains to be resolved. ";
                $message .= "This usually indicates it is less a network connectivity related issue and a higher chance of it being ";
                $message .= "a server or sensor management tool related issue.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will continue to monitor the progress of this issue, however this should be reviewed as soon as ";
                $message .= "possible in the event there is a serious or critical network or system related issue.\n\n";
                $message .= "Additional email warnings will be sent shortly if this problem is not resolved.\n\n";
                $message .= "---\nCommunication Failure Count: " . $objSensor->getSensorVar("countCommFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}    		
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countCommFailure", 111);

		} else if ($objSensor->getSensorVar("countCommFailure") >= 400 && $objSensor->getSensorVar("countCommFailure") <= 410) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);
    		
    		// Email
    		if ($objSensor->getSensorVar("emailOnCommFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnCommFailure"));
				$subject = "SMT Communication Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 3 of 4:\n---\n\nProblem:\n";
				$message .= "A communication issue with sensor '" . $objSensor->getSensorVar("name") . "' has yet to be remedied. ";
                $message .= "This prolonged communication outage usually indicates the server is unavailable or the sensor management tools ";
                $message .= "have stopped or failed.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will continue to monitor the progress of this issue, however this should be reviewed as soon as ";
                $message .= "possible in the event there is a critical system related issue going on unnoticed.\n\n";
                $message .= "One more email warning will be sent as a reminder to get this problem resolved.\n\n";
                $message .= "---\nCommunication Failure Count: " . $objSensor->getSensorVar("countCommFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}      		
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countCommFailure", 411);

		} else if ($objSensor->getSensorVar("countCommFailure") >= 800 && $objSensor->getSensorVar("countCommFailure") <= 810) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);
    		
    		// Email
    		if ($objSensor->getSensorVar("emailOnCommFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnCommFailure"));
				$subject = "SMT Communication Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 4 of 4:\n---\n\nProblem:\n";
				$message .= "This is your last reminder / warning email.\n\n";
				$message .= "The communication issue with sensor '" . $objSensor->getSensorVar("name") . "' continues to be a problem. ";
                $message .= "This prolonged communication outage will affect sensor management functionality if not resolved soon.\n\n";
				$message .= "Solution:\n";
                $message .= "Human intervention is required to identify the specific cause of this communication outage and to resolve it ";
                $message .= "promptly. This is your last email concerning this sensor communication related issue at this time.\n\n";
                $message .= "---\nCommunication Failure Count: " . $objSensor->getSensorVar("countCommFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}      		
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countCommFailure", 811);
		}    		
    }
	
    procWrite("D", ".. Analysis Processing Finished");
}

function indexer() {

	// Begin processing syslog
	procWrite("C", ".. Index Processing Started");	    

	// Instantiate new indexer object
    $indexer = new indexer();    

    // Ensure proper permissions for store
    chmod($indexer->getVar("storePath"), 0777);
    
    // Processes Indexes
    $indexer->indexRun();

    // End
    procWrite("C", " . Index Processing Finished");
}

function processReportSchedule() {
    global $ops, $login, $idsDB;

    procWrite("B", ".. Processing All Reports");

    $currentEpoch = date("U");
    $todayDate = date("y-m-d");
    $todayDateName = substr(date("l"), 0, 3);
    $todayEpoch = strtotime($todayDate);
    $tomorrowEpoch = strtotime($todayDate) + 86400;

    $dayQuery = strtolower("$todayDateName = 1");

    $idsQueryA = $idsDB->query("select id, type, status, userid, name, store, email, query from idsReportSchedule where enabled = 1 and type = 1 and status != 2 order by id desc");
    while (list($id, $type, $status, $userid, $name, $store, $email, $query) = mysql_fetch_row($idsQueryA)) {

		// Run now reports

	    $runtime = date("U");
		
		// Set status of report
	    $idsQueryB = $idsDB->query("update idsReportSchedule set data = '" . addslashes($reportData) . "', status = 1 where id = $id");
		
		$login->setSessionVar("backgroundUser", "$userid");
		$login->initSession($userid, "");
		$login->loadSession();
		
		$mySearch = new mySearch();
		$mySearch->setVar("store", "$store");
		$mySearch->setVar("query", "rqueue: $query");
		
		$reportData = $mySearch->searchDisplay();

	    $donetime = date("U");
		
		// Set status of report
	    $idsQueryB = $idsDB->query("update idsReportSchedule set data = '" . addslashes($reportData) . "', status = 2, runlast = '$donetime' where id = $id");

		// Clear cache
		clearAllStatCache();
		clearAllQueryCache();
	    
        procWrite("B", " . Report Generated [Now: $id]");
            
    }
    	    
    $idsQueryA = $idsDB->query("select id, type, status, userid, name, store, email, query, runtime, runlast from idsReportSchedule where $dayQuery and enabled = 1 and type = 2 and runlast not between $todayEpoch and $tomorrowEpoch order by id desc");
    while (list($id, $type, $status, $userid, $name, $store, $email, $query, $runtime, $runlast) = mysql_fetch_row($idsQueryA)) {

		// Scheduled report
		
		$tmpEpoch = strtotime($todayDate . " $runtime");
		if ($tmpEpoch < $currentEpoch) {

            procWrite("B", " . Sending Report [$name]");
            procWrite("B", " . Sending To [$email]");

			$login->setSessionVar("backgroundUser", "$userid");
			$login->initSession($userid, "");
			$login->loadSession();
			
			$mySearch = new mySearch();
			$mySearch->setVar("store", "$store");
			$mySearch->setVar("query", "rqueue: $query");
			
			$reportData = $mySearch->searchDisplay();
            
			// Set status of report
		    $idsQueryB = $idsDB->query("update idsReportSchedule set data = '" . addslashes($reportData) . "', status = 2, runlast = '$currentEpoch' where id = $id");
			
            // Send Reports
            $fromEmail = $email;
            $toEmail = $email;
            $fromName = "" . ops("site_consoleName") . " Console";
            $toName = "" . ops("site_consoleName") . " Report Recipient";

	    	emailPDF(reportParsePDF($reportData, 1), $email, $email, "Report: \"$name\"");
		    
            dbLog("Reports", "Sending Custom report", $flag, $toEmail, "");
    	}
    	
    }

    if (!$report && !$reportData) {
        // Nothing to do
        procWrite("B", " . Nothing to do");
    }

    procWrite("B", " . Processing Finished");
    
}

function processDailyReports() {
	global $login;

    procWrite("B", ".. Processing Stats Report");

    $currentEpoch = date("U");
    $todayDate = date("y-m-d");
    $todayEpoch = strtotime($todayDate);
    $tomorrowEpoch = strtotime($todayDate + 86400);
    $reportSendTime = ops("site_dailyStatsReportTime");
	$reportSendEpoch = strtotime("$todayDate $reportSendTime");
    $reportLastSendTime = ops("site_dailyStatsReportLast");

    if (ops("site_dailyStatsReportEmail") != "") {
        // Generate and send report to the listed emails

        // Check if ran yet today
        if ($reportLastSendTime < $todayEpoch) {

            // Only run at specified time
            if ($currentEpoch > $reportSendEpoch) {

                procWrite("B", " . Sending Reports");

				$login->setSessionVar("backgroundUser", "102");
				$login->initSession(102, "");
				$login->loadSession();
                
				$mySearch = new mySearch();
#				$mySearch->setVar("store", ops("site_activeIndexStore"));
				$mySearch->setVar("query", "returnid: today:");
				$tmp = $mySearch->searchDisplay();
                $statsToday = count($tmp);

				$mySearch = new mySearch();
#				$mySearch->setVar("store", ops("site_activeIndexStore"));
				$mySearch->setVar("query", "returnid: yesterday:");
				$tmp = $mySearch->searchDisplay();
                $statsYesterday = count($tmp);
                
                echo "Count: $statsYesterday\n";
                
                // Get Status for each processor
                $objStatus = new myStatus();
                $objStatus->loadStatus();

            	$processorALastRun = statusProcessorLastRun("A");
            	$processorALastRunDiff = statusProcessorLastRunDiff("A");
            	$processorARunCount = statusProcessorRunCount("A");

            	$processorBLastRun = statusProcessorLastRun("B");
            	$processorBLastRunDiff = statusProcessorLastRunDiff("B");
            	$processorBRunCount = statusProcessorRunCount("B");

            	$processorCLastRun = statusProcessorLastRun("C");
            	$processorCLastRunDiff = statusProcessorLastRunDiff("C");
            	$processorCRunCount = statusProcessorRunCount("C");

            	$processorDLastRun = statusProcessorLastRun("D");
            	$processorDLastRunDiff = statusProcessorLastRunDiff("D");
            	$processorDRunCount = statusProcessorRunCount("D");

                // Assess Status for each processor
            	if ($processorALastRunDiff > 1000) {
            		$processorAStatus = "Failure (hung or not running)";
            	} else if ($processorALastRunDiff > 480) {
                    $processorAStatus = "Danger (might be a problem)";
                } else if ($processorALastRunDiff > 240) {
                    $processorAStatus = "Warning (taking longer than usual)";
            	} else {
                    $processorAStatus = "Active";
            	}

            	if ($processorBLastRunDiff > 1000) {
            		$processorBStatus = "Failure (hung or not running)";
            	} else if ($processorBLastRunDiff > 480) {
                    $processorBStatus = "Danger (might be a problem)";
                } else if ($processorBLastRunDiff > 240) {
                    $processorBStatus = "Warning (taking longer than usual)";
            	} else {
                    $processorBStatus = "Active";
            	}

            	if ($processorCLastRunDiff > 1000) {
            		$processorCStatus = "Failure (hung or not running)";
            	} else if ($processorCLastRunDiff > 480) {
                    $processorCStatus = "Danger (might be a problem)";
                } else if ($processorCLastRunDiff > 240) {
                    $processorCStatus = "Warning (taking longer than usual)";
            	} else {
                    $processorCStatus = "Active";
            	}

            	if ($processorDLastRunDiff > 1000) {
            		$processorDStatus = "Failure (hung or not running)";
            	} else if ($processorDLastRunDiff > 480) {
                    $processorDStatus = "Danger (might be a problem)";
                } else if ($processorDLastRunDiff > 240) {
                    $processorDStatus = "Warning (taking longer than usual)";
            	} else {
                    $processorDStatus = "Active";
            	}

                // Send Reports
                $fromEmail = ops("site_dailyStatsReportFrom");
                $toEmail = ops("site_dailyStatsReportEmail");
                $fromName = ops("site_dailyStatsReportFrom");
                $toName = "" . ops("site_consoleName") . " Report Recipient";

                $report = "Daily Status Report\n---\n\n";
                $report .= "[Total Events by Time Period]\n";
                $report .= "Today: $statsToday\n";
                $report .= "Yesterday: $statsYesterday\n\n";

                $report .= "[Console Status]\n";
                $report .= "Background Processor A Status: $processorAStatus\n";
        		$report .= "Background Processor A Last Run: " . timeDisplay($processorALastRun, 1) . " ($processorALastRun)\n";
                $report .= "Background Processor A Last Run Diff: $processorALastRunDiff\n";
                $report .= "Background Processor A Run Count: $processorARunCount\n";
                $report .= "\n";
                $report .= "Background Processor B Status: $processorBStatus\n";
        		$report .= "Background Processor B Last Run: " . timeDisplay($processorBLastRun, 1) . " ($processorBLastRun)\n";
                $report .= "Background Processor B Last Run Diff: $processorBLastRunDiff\n";
                $report .= "Background Processor B Run Count: $processorBRunCount\n";
                $report .= "\n";
                $report .= "Background Processor C Status: $processorCStatus\n";
        		$report .= "Background Processor C Last Run: " . timeDisplay($processorCLastRun, 1) . " ($processorCLastRun)\n";
                $report .= "Background Processor C Last Run Diff: $processorCLastRunDiff\n";
                $report .= "Background Processor C Run Count: $processorCRunCount\n";
                $report .= "\n";
                $report .= "Background Processor D Status: $processorDStatus\n";
        		$report .= "Background Processor D Last Run: " . timeDisplay($processorDLastRun, 1) . " ($processorDLastRun)\n";
                $report .= "Background Processor D Last Run Diff: $processorDLastRunDiff\n";
                $report .= "Background Processor D Run Count: $processorDRunCount\n";
                $report .= "\n";

                sendmail($fromEmail, $fromName, $toEmail, $toName, ops("site_dailyStatsReportSubject"), $report);
                dbLog("Reports", "Sending Stats Report", $flag, $toEmail, "");

                // Update last sent
                ops("site_dailyStatsReportLast", $currentEpoch);
            } else {
                 procWrite("B", " . Not Quite Time");
            }
        } else {
             procWrite("B", " . Already Sent Today");
        }

    } else {
        // Nothing to do
        procWrite("B", " . No Recipients for Report");
    }

    procWrite("B", " . Processing Finished");
    
}

function autoSnortTrimmer() {
	global $ops;
	
	$objModule = new myModule();
	$objModule->loadModule("1");

	// Return if snort module is not enabled
    if ($objModule->getModuleVar("enabled") == 0) {
		return;
    }
    
    // Return if snort auto trimming is disabled
	if (ops("site_snortAutoTrim") == 0) {
		return;
	}

    procWrite("B", ".. Snort Auto Trimming Started");
	
	$moduleDB = new moduleDB;
	$moduleDB->loadDB(1);
		
	$Q = $moduleDB->query("select count(*) as cnt from event");
	if(mysql_num_rows($Q) > 0) {
		$row = mysql_fetch_array($Q);
		$eventTotal = $row["cnt"];
	}
	
	if (ops("site_snortAutoTrimMax") > $eventTotal) {
		// Nothing to be done
	    procWrite("B", " . 0 events to be trimmed");	
	} else {
	
		$diff = $eventTotal - ops("site_snortAutoTrimMax");
		
	    procWrite("B", " . $diff events to be trimmed");	
		
		if ($diff > 5000) {
			$diff = 5000;
		    procWrite("B", " . 5000 events will be trimmed");	
		}
		
		if ($diff > 0) {	
	
			$Q = $moduleDB->query("select sid, cid from event order by timestamp limit $diff");
			if(mysql_num_rows($Q) > 0) {
				while ($row = mysql_fetch_array($Q)) {
					$cnt++;
					$Q2 = $moduleDB->query("delete from event where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from data where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from tcphdr where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from udphdr where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from iphdr where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from icmphdr where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from opt where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
				}
				
			    procWrite("B", " . $cnt events trimmed this run");	
			}		
	
		} else {
			
		}
	}
	
	$moduleDB->close();	
	
    procWrite("B", " . Snort Auto Trimming Finished");
}

function checkModule($module) {

    $array = get_loaded_extensions();
    
    for ($i=0;$i<count($array);$i++) {
        if (strstr($array[$i], $module)) {
            $f = 1;
        }
    }

    if ($f == 1) {
		return 1;
    } else {
		return 0;
    }

    // should never reach here
    return 0;

}

function procWrite($processor, $data) {
	global $ops, $root_path;
	
	$data = strip_tags(addslashes($data));
	
	if ($data == "RESET") {
		// Open File and Clear
        if ($fp = @fopen("$root_path" . "tmp/processor$processor", "w")) {

        	// Write procWrite data
        	fputs ($fp, "\n");
        	
        	// Close file handle
            fclose($fp);
        }	
	} else {
		
		// Open File and Append
        if ($fp = @fopen("$root_path" . "tmp/processor$processor", "a")) {

        	// Write procWrite data
        	fputs ($fp, "$data\n");

        	// Close file handle
            fclose($fp);
        }
		
		echo "$data\n";
	}
}

?>

