<?php

// Sets
error_reporting(E_ALL ^ E_NOTICE);
ini_set("memory_limit", -1);
ini_set("display_errors", 1);

// Tell the system we are in processor mode
$processor = 1;

// Set some initital variables
$processorVersion = "4.6";
$flashWriteCount = 0;

// Includes
if (!@include "../conf/conf.php") {
	echo "Console BPU Fatal Error: This console is not properly configured. Please re-install.\n\n";
	echo "FILE: ../conf/conf.php - Missing or not found.\n\n";
	exit;
}
if (!@include "./processor.php") {
	echo "Console BPU Fatal Error: This console is not properly configured. Please re-install.\n\n";
	echo "FILE: ./processor.php - Missing or not found.\n\n";
	exit;
}

// Exit if in setup or upgrade mode
if (file_exists("../installation/setup.php")) {
	echo "The console is in setup / upgrade mode - Exiting.\n\n";
	exit;
}

// php module checking
$module = "mysql";
if (checkModule("$module") == 0) {
	procWrite("A", "----------------------------------------------------");
	procWrite("A", ":: BPU: Background Processing Unit A v$processorVersion");
	procWrite("A", "----------------------------------------------------\n");
	procWrite("A", ".! Fatal Error: $module module not available.\n");
	procwrite("A", "The required module [$module] is not available within your php binary (cgi).");
	procwrite("A", "You must recompile your php binary (cgi) with $module support.");
	procwrite("A", "This must be corrected before the console will operate correctly.\n");
	exit;
}
$module = "pcre";
if (checkModule("$module") == 0) {
	procWrite("A", "----------------------------------------------------");
	procWrite("A", ":: BPU: Background Processing Unit A v$processorVersion");
	procWrite("A", "----------------------------------------------------\n");
	procWrite("A", ".! Fatal Error: $module module not available.\n");
	procwrite("A", "The required module [$module] is not available within your php binary (cgi).");
	procwrite("A", "You must recompile your php binary (cgi) with $module support.");
	procwrite("A", "This must be corrected before the console will operate correctly.\n");
	exit;
}

// Load appropriate modules
recursiveModuleLoad(0);

// Load Database Connections
$idsDB = new idsDB;

// Simulate login to console
$login = new mySession("processor");

// Set Script TIMEOUT Value
set_time_limit(2000);

// Variable Initilization
$ops = new myOps();
$ops->loadOps();

function aLoop() {
	global $root_path, $processorVersion, $flashWriteCount, $masterCnt;

	// Start Timer
	$timerStart = date("U");
	
	procWrite("A", "RESET");
	procWrite("A", "----------------------------------------------------");
	procWrite("A", ":: Background Processing Unit A v$processorVersion");
	procWrite("A", ":: " . versionText());
	procWrite("A", ":: Date & Time " . timeDisplay($timerStart, 1));
	procWrite("A", ":: Finished " . statusProcessorLastRunDiff("A") . " seconds ago");
	procWrite("A", "----------------------------------------------------");
	
	// Hold Check
	if (file_exists("$root_path/tmp/holdA.tmp")) { procwrite("A", ":: Processor On Hold -> Exiting"); exit; }
	
	// Customized Operations Loading
	customOperations();

	// Integrity Checking
	tableIntegrity();
	
	// Snort Processing Function
	idsSnort();
	
	// Syslog Processing Function
	idsSyslog();
	
	// Status Setting
	$objStatus = new myStatus();
	$objStatus->loadStatus();
	$objStatus->setStatusVar("processorA_lastRun", date("U"));
	$objStatus->totalProcessorRunIncrement("A");

	// Timer Calculations
	$timerDiff = date("U") - $timerStart;
	
	// Write Flash Monitor Data
	writeFlashPoll($flashWriteCount);
	writeImportCalculator($flashWriteCount, $timerDiff+3);
	$flashWriteCount = 0;
	
	// Footer
	procWrite("A", "----------------------------------------------------");
	procWrite("A", ":: Iteration $masterCnt");
	procWrite("A", ":: Completed in $timerDiff seconds");
	procWrite("A", "----------------------------------------------------");

}

while ($masterCnt != ops("site_bpuIterationA")) {
	$masterCnt++;

	// Perform Loop
	aLoop();
	
	// Sleep
	sleep(ops("site_bpuSleepA"));
}

// Close Database Connections
$idsDB->close();

exit;

?>
