<?php

// Sets
error_reporting(E_ALL ^ E_NOTICE);
ini_set("memory_limit", -1);
ini_set("display_errors", 1);

// Tell the system we are in processor mode
$processor = 1;

// Set some initital variables
$processorVersion = "5.1";

// Includes
if (!@include "../conf/conf.php") {
	echo "Console BPU Fatal Error: This console is not properly configured. Please re-install.\n\n";
	echo "FILE: ../conf/conf.php - Missing or not found.\n\n";
	exit;
}
if (!@include "./processor.php") {
	echo "Console BPU Fatal Error: This console is not properly configured. Please re-install.\n\n";
	echo "FILE: ./processor.php - Missing or not found.\n\n";
	exit;
}

// Exit if in setup or upgrade mode
if (file_exists("../installation/setup.php")) {
	echo "The console is in setup / upgrade mode - Exiting.\n\n";
	exit;
}

// php module checking
$module = "mysql";
if (checkModule("$module") == 0) {
	procWrite("B", "----------------------------------------------------");
	procWrite("B", ":: BPU: Background Processing Unit B v$processorVersion");
	procWrite("B", "----------------------------------------------------\n");
	procWrite("B", ".! Fatal Error: $module module not available.\n");
	procwrite("B", "The required module [$module] is not available within your php binary (cgi).");
	procwrite("B", "You must recompile your php binary (cgi) with $module support.");
	procwrite("B", "This must be corrected before console will operate correctly.\n");
	exit;
}
$module = "pcre";
if (checkModule("$module") == 0) {
	procWrite("B", "----------------------------------------------------");
	procWrite("B", ":: BPU: Background Processing Unit B v$processorVersion");
	procWrite("B", "----------------------------------------------------\n");
	procWrite("B", ".! Fatal Error: $module module not available.\n");
	procwrite("B", "The required module [$module] is not available within your php binary (cgi).");
	procwrite("B", "You must recompile your php binary (cgi) with $module support.");
	procwrite("B", "This must be corrected before console will operate correctly.\n");
	exit;
}

$module = "gd";
if (checkModule("$module") == 0) {
	procWrite("B", "----------------------------------------------------");
	procWrite("B", ":: BPU: Background Processing Unit B v$processorVersion");
	procWrite("B", "----------------------------------------------------\n");
	procWrite("B", ".! Critical Warning: $module module not available.\n");
	procwrite("B", "The required module [$module] is not available within your php binary (cgi).");
	procwrite("B", "You must recompile your php binary (cgi) with $module support.");
	procwrite("B", "This must be corrected or you may experience operational problems.\n");
}

// Load appropriate modules
recursiveModuleLoad(0);

// Load Database Connections
$idsDB = new idsDB;

// Simulate login to console
$login = new mySession("processor");

// Set Script TIMEOUT Value
set_time_limit(2000);

// Variable Initilization
$ops = new myOps();
$ops->loadOps();
	
function bLoop() {
	global $root_path, $processorVersion, $flashWriteCount, $masterCnt;

	// Away we go!
	$timerStart = date("U");
	
	procWrite("B", "RESET");
	
	procWrite("B", "----------------------------------------------------");
	procWrite("B", ":: Background Processing Unit B v$processorVersion");
	procWrite("B", ":: " . versionText());
	procWrite("B", ":: Date & Time " . timeDisplay($timerStart, 1));
	procWrite("B", ":: Finished " . statusProcessorLastRunDiff("B") . " seconds ago");
	procWrite("B", "----------------------------------------------------");
	
	// Hold Check
	if (file_exists("$root_path/tmp/holdB.tmp")) { procwrite("B", ":: ON HOLD -> Exiting");	exit; }
	
	// Customized Operations Loading
	customOperations();	
	
	// Permissions Checking
	permissionsCheck();
	
	// Version Checking
	remoteVersionCheck();
	
	// Community Communication
	communityBufferProcess();

	// System Status
	gatherSystemStatus("uptime", "w");
	gatherSystemStatus("df", "w");
	gatherSystemStatus("load_history", "a");
	
	// Deletions
	deleteDataProcessor();
	
	// Scanner
	scanProcessor();
	
	// Signature System
	sigProcessor(ops("site_sigFile"), 1);
	sigProcessor(ops("site_sigFileAlt"), 2);

	// Auto Snort Trimming
	autoSnortTrimmer();
	
	// Daily Reports
	processDailyReports();
	
	// Report Schedule
	processReportSchedule();
	
	// Hostname Resolving
	hostnameProcessor();
	
	$objStatus = new myStatus();
	$objStatus->loadStatus();
	$objStatus->setStatusVar("processorB_lastRun", date("U"));
	$objStatus->totalProcessorRunIncrement("B");
	
	$timerDiff = date("U") - $timerStart;
	
	// Footer
	procWrite("B", "----------------------------------------------------");
	procWrite("B", ":: Iteration $masterCnt");
	procWrite("B", ":: Completed in $timerDiff seconds");
	procWrite("B", "----------------------------------------------------");
}

while ($masterCnt != ops("site_bpuIterationB")) {
	$masterCnt++;

	// Perform Loop
	bLoop();
	
	// Sleep
	sleep(ops("site_bpuSleepB"));
}

// Close Database Connections
$idsDB->close();

exit;

?>
